@@ -13,7 +13,7 @@ def patch_file(path, new_version):
     assert ".dev" in new_version
     new_version = new_version.lstrip("v")
 
-    with open(path, "r") as fin:
+    with open(path) as fin:
         lines = fin.readlines()
 
     for i, line in enumerate(lines, start=0):
