@@ -1,5 +1,3 @@
-#!/usr/bin/env python
-#
 # This file describes eFuses for ESP32 chip
 #
 # SPDX-FileCopyrightText: 2020-2022 Espressif Systems (Shanghai) CO LTD
@@ -11,6 +9,7 @@ import struct
 import time
 
 import esptool
+from esptool import log
 
 from .mem_definition import EfuseDefineBlocks, EfuseDefineFields, EfuseDefineRegisters
 from .. import base_fields
@@ -28,7 +27,7 @@ class EfuseBlock(base_fields.EfuseBlockBase):
         else:
             if parent.coding_scheme is None:
                 parent.read_coding_scheme()
-        super(EfuseBlock, self).__init__(parent, param, skip_read=skip_read)
+        super().__init__(parent, param, skip_read=skip_read)
 
     def apply_coding_scheme(self):
         data = self.get_raw(from_read=False)[::-1]
@@ -67,23 +66,23 @@ class EspEfuses(base_fields.EspEfusesBase):
     Wrapper object to manage the efuse fields in a connected ESP bootloader
     """
 
-    Blocks = EfuseDefineBlocks()
-    Fields = EfuseDefineFields()
-    REGS = EfuseDefineRegisters
-    BURN_BLOCK_DATA_NAMES = Blocks.get_burn_block_data_names()
-    BLOCKS_FOR_KEYS = Blocks.get_blocks_for_keys()
-
-    debug = False
-    do_not_confirm = False
-
-    def __init__(self, esp, skip_connect=False, debug=False, do_not_confirm=False):
-        self._esp = esp
-        self.debug = debug
-        self.do_not_confirm = do_not_confirm
+    def __init__(
+        self,
+        esp,
+        skip_connect=False,
+        debug=False,
+        do_not_confirm=False,
+        extend_efuse_table=None,
+    ):
+        super().__init__(esp, skip_connect, debug, do_not_confirm, extend_efuse_table)
+        self.Blocks = EfuseDefineBlocks()
+        self.Fields = EfuseDefineFields(extend_efuse_table)
+        self.REGS = EfuseDefineRegisters
+        self.BURN_BLOCK_DATA_NAMES = self.Blocks.get_burn_block_data_names()
+        self.BLOCKS_FOR_KEYS = self.Blocks.get_blocks_for_keys()
         if esp.CHIP_NAME != "ESP32":
             raise esptool.FatalError(
-                "Expected the 'esp' param for ESP32 chip but got for '%s'."
-                % (esp.CHIP_NAME)
+                f"Expected the 'esp' param for ESP32 chip but got for '{esp.CHIP_NAME}'."
             )
         self.blocks = [
             EfuseBlock(self, self.Blocks.get(block), skip_read=skip_connect)
@@ -91,112 +90,87 @@ class EspEfuses(base_fields.EspEfusesBase):
         ]
         if not skip_connect:
             self.get_coding_scheme_warnings()
-        self.efuses = [
-            EfuseField.from_tuple(
-                self, self.Fields.get(efuse), self.Fields.get(efuse).class_type
-            )
-            for efuse in self.Fields.EFUSES
-        ]
+        self.efuses = [EfuseField.convert(self, efuse) for efuse in self.Fields.EFUSES]
         if skip_connect:
             self.efuses += [
-                EfuseField.from_tuple(
-                    self, self.Fields.get(efuse), self.Fields.get(efuse).class_type
-                )
-                for efuse in self.Fields.KEYBLOCKS_256
+                EfuseField.convert(self, efuse) for efuse in self.Fields.KEYBLOCKS_256
             ]
             self.efuses += [
-                EfuseField.from_tuple(
-                    self, self.Fields.get(efuse), self.Fields.get(efuse).class_type
-                )
-                for efuse in self.Fields.CUSTOM_MAC
+                EfuseField.convert(self, efuse) for efuse in self.Fields.CUSTOM_MAC
             ]
             self.efuses += [
-                EfuseField.from_tuple(
-                    self, self.Fields.get(efuse), self.Fields.get(efuse).class_type
-                )
-                for efuse in self.Fields.ADC_CALIBRATION
+                EfuseField.convert(self, efuse) for efuse in self.Fields.ADC_CALIBRATION
             ]
         else:
             if self.coding_scheme == self.REGS.CODING_SCHEME_NONE:
                 self.efuses += [
-                    EfuseField.from_tuple(
-                        self, self.Fields.get(efuse), self.Fields.get(efuse).class_type
-                    )
+                    EfuseField.convert(self, efuse)
                     for efuse in self.Fields.KEYBLOCKS_256
                 ]
             elif self.coding_scheme == self.REGS.CODING_SCHEME_34:
                 self.efuses += [
-                    EfuseField.from_tuple(
-                        self, self.Fields.get(efuse), self.Fields.get(efuse).class_type
-                    )
+                    EfuseField.convert(self, efuse)
                     for efuse in self.Fields.KEYBLOCKS_192
                 ]
             else:
                 raise esptool.FatalError(
-                    "The coding scheme (%d) - is not supported" % self.coding_scheme
+                    f"The coding scheme ({self.coding_scheme}) - is not supported"
                 )
             if self["MAC_VERSION"].get() == 1:
                 self.efuses += [
-                    EfuseField.from_tuple(
-                        self, self.Fields.get(efuse), self.Fields.get(efuse).class_type
-                    )
-                    for efuse in self.Fields.CUSTOM_MAC
+                    EfuseField.convert(self, efuse) for efuse in self.Fields.CUSTOM_MAC
                 ]
             if self["BLK3_PART_RESERVE"].get():
                 self.efuses += [
-                    EfuseField.from_tuple(
-                        self, self.Fields.get(efuse), self.Fields.get(efuse).class_type
-                    )
+                    EfuseField.convert(self, efuse)
                     for efuse in self.Fields.ADC_CALIBRATION
                 ]
             self.efuses += [
-                EfuseField.from_tuple(
-                    self, self.Fields.get(efuse), self.Fields.get(efuse).class_type
-                )
-                for efuse in self.Fields.CALC
+                EfuseField.convert(self, efuse) for efuse in self.Fields.CALC
             ]
 
     def __getitem__(self, efuse_name):
         """Return the efuse field with the given name"""
         for e in self.efuses:
-            if efuse_name == e.name:
+            if efuse_name == e.name or any(x == efuse_name for x in e.alt_names):
                 return e
         new_fields = False
         for efuse in self.Fields.CUSTOM_MAC:
-            e = self.Fields.get(efuse)
-            if e.name == efuse_name:
+            if efuse.name == efuse_name or any(
+                x == efuse_name for x in efuse.alt_names
+            ):
                 self.efuses += [
-                    EfuseField.from_tuple(
-                        self, self.Fields.get(efuse), self.Fields.get(efuse).class_type
-                    )
-                    for efuse in self.Fields.CUSTOM_MAC
+                    EfuseField.convert(self, efuse) for efuse in self.Fields.CUSTOM_MAC
                 ]
                 new_fields = True
         for efuse in self.Fields.ADC_CALIBRATION:
-            e = self.Fields.get(efuse)
-            if e.name == efuse_name:
+            if efuse.name == efuse_name or any(
+                x == efuse_name for x in efuse.alt_names
+            ):
                 self.efuses += [
-                    EfuseField.from_tuple(
-                        self, self.Fields.get(efuse), self.Fields.get(efuse).class_type
-                    )
+                    EfuseField.convert(self, efuse)
                     for efuse in self.Fields.ADC_CALIBRATION
                 ]
                 new_fields = True
         if new_fields:
             for e in self.efuses:
-                if efuse_name == e.name:
+                if efuse_name == e.name or any(x == efuse_name for x in e.alt_names):
                     return e
         raise KeyError
 
     def read_coding_scheme(self):
-        self.coding_scheme = (
+        coding_scheme = (
             self.read_efuse(self.REGS.EFUSE_CODING_SCHEME_WORD)
             & self.REGS.EFUSE_CODING_SCHEME_MASK
         )
+        if coding_scheme == self.REGS.CODING_SCHEME_NONE_RECOVERY:
+            self.coding_scheme = self.REGS.CODING_SCHEME_NONE
+        else:
+            self.coding_scheme = coding_scheme
 
     def print_status_regs(self):
-        print("")
-        print(
+        log.print("")
+        log.print(
             "{:27} 0x{:08x}".format(
                 "EFUSE_REG_DEC_STATUS", self.read_reg(self.REGS.EFUSE_REG_DEC_STATUS)
             )
@@ -233,7 +207,7 @@ class EspEfuses(base_fields.EspEfusesBase):
             if self.read_reg(self.REGS.EFUSE_REG_CMD) == 0:
                 return
         raise esptool.FatalError(
-            "Timed out waiting for Efuse controller command to complete"
+            "Timed out waiting for eFuse controller command to complete"
         )
 
     def efuse_read(self):
@@ -255,9 +229,9 @@ class EspEfuses(base_fields.EspEfusesBase):
                 block.num_errors = 0
                 block.fail = err != 0
             if not silent and block.fail:
-                print(
-                    "Error(s) in BLOCK%d [ERRORS:%d FAIL:%d]"
-                    % (block.id, block.num_errors, block.fail)
+                log.print(
+                    f"Error(s) in BLOCK{block.id} "
+                    f"[ERRORS:{block.num_errors} FAIL:{block.fail}]"
                 )
         if (self.debug or err) and not silent:
             self.print_status_regs()
@@ -265,8 +239,7 @@ class EspEfuses(base_fields.EspEfusesBase):
 
     def summary(self):
         if self["XPD_SDIO_FORCE"].get() == 0:
-            output = "Flash voltage (VDD_SDIO) determined by GPIO12 on reset "
-            "(High for 1.8V, Low/NC for 3.3V)."
+            output = "Flash voltage (VDD_SDIO) determined by GPIO12 on reset (High for 1.8V, Low/NC for 3.3V)"
         elif self["XPD_SDIO_REG"].get() == 0:
             output = "Flash voltage (VDD_SDIO) internal regulator disabled by efuse."
         elif self["XPD_SDIO_TIEH"].get() == 0:
@@ -278,7 +251,7 @@ class EspEfuses(base_fields.EspEfusesBase):
 
 class EfuseField(base_fields.EfuseFieldBase):
     @staticmethod
-    def from_tuple(parent, efuse_tuple, type_class):
+    def convert(parent, efuse):
         return {
             "mac": EfuseMacField,
             "spipin": EfuseSpiPinField,
@@ -286,10 +259,7 @@ class EfuseField(base_fields.EfuseFieldBase):
             "adc_tp": EfuseAdcPointCalibration,
             "wafer": EfuseWafer,
             "pkg": EfusePkg,
-        }.get(type_class, EfuseField)(parent, efuse_tuple)
-
-    def get_info(self):
-        return "%s (BLOCK%d):" % (self.name, self.block)
+        }.get(efuse.class_type, EfuseField)(parent, efuse)
 
 
 class EfuseMacField(EfuseField):
@@ -298,7 +268,7 @@ class EfuseMacField(EfuseField):
     (if MAC_VERSION == 1 then the CUSTOM_MAC is used)
     """
 
-    def check_format(self, new_value_str):
+    def check_format(self, new_value_str: str | None):
         if new_value_str is None:
             raise esptool.FatalError(
                 "Required MAC Address in AA:CD:EF:01:02:03 format!"
@@ -326,13 +296,12 @@ class EfuseMacField(EfuseField):
     def get_and_check(raw_mac, stored_crc):
         computed_crc = EfuseMacField.calc_crc(raw_mac)
         if computed_crc == stored_crc:
-            valid_msg = "(CRC 0x%02x OK)" % stored_crc
+            valid_msg = f"(CRC {stored_crc:#04x} OK)"
         else:
-            valid_msg = "(CRC 0x%02x invalid - calculated 0x%02x)" % (
-                stored_crc,
-                computed_crc,
+            valid_msg = (
+                f"(CRC {stored_crc:#04x} invalid - calculated {computed_crc:#04x})"
             )
-        return "%s %s" % (util.hexify(raw_mac, ":"), valid_msg)
+        return " ".join([util.hexify(raw_mac, ":"), valid_msg])
 
     @staticmethod
     def calc_crc(raw_mac):
@@ -362,10 +331,8 @@ class EfuseMacField(EfuseField):
 
     def save(self, new_value):
         def print_field(e, new_value):
-            print(
-                "    - '{}' ({}) {} -> {}".format(
-                    e.name, e.description, e.get_bitstring(), new_value
-                )
+            log.print(
+                f"    - '{e.name}' ({e.description}) {e.get_bitstring()} -> {new_value}"
             )
 
         if self.name == "CUSTOM_MAC":
@@ -382,14 +349,12 @@ class EfuseMacField(EfuseField):
                 if mac_version.get() != 1:
                     if not self.parent.force_write_always:
                         raise esptool.FatalError(
-                            "MAC_VERSION = {}, should be 0 or 1.".format(
-                                mac_version.get()
-                            )
+                            f"MAC_VERSION = {mac_version.get()}, should be 0 or 1."
                         )
 
             bitarray_mac = self.convert_to_bitstring(new_value)
             print_field(self, bitarray_mac)
-            super(EfuseMacField, self).save(new_value)
+            super().save(new_value)
 
             crc_val = self.calc_crc(new_value)
             crc_field = self.parent["CUSTOM_MAC_CRC"]
@@ -404,7 +369,9 @@ class EfuseMacField(EfuseField):
 class EfuseWafer(EfuseField):
     def get(self, from_read=True):
         rev_bit0 = self.parent["CHIP_VER_REV1"].get(from_read)
+        assert self.parent["CHIP_VER_REV1"].bit_len == 1
         rev_bit1 = self.parent["CHIP_VER_REV2"].get(from_read)
+        assert self.parent["CHIP_VER_REV2"].bit_len == 1
         apb_ctl_date = self.parent.read_reg(self.parent.REGS.APB_CTL_DATE_ADDR)
         rev_bit2 = (
             apb_ctl_date >> self.parent.REGS.APB_CTL_DATE_S
@@ -420,7 +387,7 @@ class EfuseWafer(EfuseField):
         return revision
 
     def save(self, new_value):
-        raise esptool.FatalError("Burning %s is not supported" % self.name)
+        raise esptool.FatalError(f"Burning {self.name} is not supported")
 
 
 class EfusePkg(EfuseField):
@@ -430,7 +397,7 @@ class EfusePkg(EfuseField):
         return (hi_bits << 3) + lo_bits
 
     def save(self, new_value):
-        raise esptool.FatalError("Burning %s is not supported" % self.name)
+        raise esptool.FatalError(f"Burning {self.name} is not supported.")
 
 
 class EfuseSpiPinField(EfuseField):
@@ -452,8 +419,7 @@ class EfuseSpiPinField(EfuseField):
             )
         elif new_value_int > 33:
             raise esptool.FatalError(
-                "IO pin %d cannot be set for SPI flash. 0-29, 32 & 33 only."
-                % new_value_int
+                f"IO pin {new_value_int} cannot be set for SPI flash. 0-29, 32 & 33 only."
             )
         elif new_value_int in [32, 33]:
             return str(new_value_int - 2)
