@@ -97,10 +97,7 @@ class EfuseDefineFields(EfuseFieldsBase):
     def __init__(self, extend_efuse_table) -> None:
         # List of efuse fields from TRM the chapter eFuse Controller.
         self.EFUSES = []
-
         self.KEYBLOCKS = []
-
-        # if BLK_VERSION_MINOR is 1, these efuse fields are in BLOCK2
         self.BLOCK2_CALIBRATION_EFUSES = []
 
         dir_name = os.path.dirname(os.path.abspath(__file__))
@@ -108,7 +105,7 @@ class EfuseDefineFields(EfuseFieldsBase):
         file_name = file_name + ".yaml"
         dir_name, _ = os.path.split(dir_name)
         efuse_file = os.path.join(dir_name, "efuse_defs", file_name)
-        with open(f"{efuse_file}", "r") as r_file:
+        with open(f"{efuse_file}") as r_file:
             e_desc = yaml.safe_load(r_file)
         super().__init__(e_desc, extend_efuse_table)
 
